log using chapter2.log, replace

******* Chapter 2 *******

***** Chapter 2.1 Introduction 

* simulated data set "University" *
* the simulated data are stored as multidimensional frequency table in the
* EXCEL-sheet: university.xlsx 
* with variables (named in row 1) 
* in columns  A: y  (attending university; 1=yes, 0=no)
*             B: x1 (country; 0=A 1=B)
*             C: x2 (academic background; 1=yes 0=n0)
*             D: x3 (sibling 0=no or one, 1=two or more)
*             E: x4 (intelleigence score; quintiles)
*             F: f  (frequencies (weight); number of cases in group)

* importing and preparing data from excel sheet for University data
* note: excel sheet have to be in same directory as STATA command do-file
*       (otherwise: full path is necessary)
import excel "university.xlsx", sheet("data") firstrow clear
*
* setting variable and value labels
label var y "U: attending university"
label define y 0 "not attending" 1 "attending"
label value y y 
*
label var x1  "C country"
label def x1 0 "A" 1 "B"
label val x1 x1
*
label var x2 "A: academic background of parents"
label def x2 0 "no" 1 "yes"
label val x2 x2
*
label var x3 "S: sibling"
label def x3 0 "no or one" 1 "two or more"
label val x3 x3

*
label var x4 "I: quintiles from N(0,15) intelligence scores"
*
label var f "frequencies (weights)"
*
* Note: 
* "university.xlsx" is aggregated over all 80 combinations (cells) of the five 
* variables. the variable "freq" gives the frequencies of each combination and 
* have to be included into STATA-regression commands for single cases as weight 
* ("[fw=f]" or[iw=f], where [iw=...] allows not only intager but also real
* numbers). Alternatively, expand aggregated frequencies to 2,000,000 singular 
* cases using Stata command “expand freq” (but them file becomes larger from
* <1KB to 19.1 MB:
*

***** TABLES *****

*** Table 2.1 : Logistic, Probit and LPM effects on University Attendance (Y=1)
*
** Computations
* logistic effects (beta) saved in scalars
logit y x1-x4 [fw=f],or cformat(%6.3f)
logit , cformat(%6.3f)
scalar betaC_ASI=_b[x1]
scalar betaA_CSI=_b[x2]
scalar betaS_CAI=_b[x3]
scalar betaI_CAS=_b[x4]
scalar betaU=_b[_cons]
*
* probit effects (gamma)
probit y x1-x4 [fw=f] ,cformat(%6.3f)
scalar gammaC_ASI=_b[x1]
scalar gammaA_CSI=_b[x2]
scalar gammaS_CAI=_b[x3]
scalar gammaI_CAS=_b[x4]
scalar gammaU=_b[_cons]
*
* PLM effects (alpha)
reg y x1-x4 [fw=f] ,cformat(%6.3f) 
scalar alphaC_ASI=_b[x1]
scalar alphaA_CSI=_b[x2]
scalar alphaS_CAI=_b[x3]
scalar alphaI_CAS=_b[x4]
scalar alphaU=_b[_cons]
*
** table 2.1
display "Logistic effects:"
display "Country (1=B)  OR = " %5.3f exp(betaC_ASI) "; beta = " %6.3f betaC_ASI
display "Acad. (1=yes)  OR = " %5.3f exp(betaA_CSI) "; beta = " %6.3f betaA_CSI
display "Sibling (1=2+) OR = " %5.3f exp(betaS_CAI) "; beta = " %6.3f betaS_CAI
display "Intelligence   OR = " %5.3f exp(betaI_CAS) "; beta = " %6.3f betaI_CAS
display "Constant       OR = " %5.3f exp(betaU) ";     beta = " %6.3f betaU
*
display "probit effects:"
display "Country (1=B)  gamma = " %6.3f gammaC_ASI
display "Acad. (1=yes)  gamma = " %6.3f gammaA_CSI
display "Sibling (1=2+) gamma = " %6.3f gammaS_CAI
display "Intelligence   gamma = " %6.3f gammaI_CAS
display "Constant       gamma = " %6.3f gammaU
*
display "LPM effects:"
display "Country (1=B)  alpha = " %6.3f alphaC_ASI
display "Acad. (1=yes)  alpha = " %6.3f alphaA_CSI
display "Sibling (1=2+) alpha = " %6.3f alphaS_CAI
display "Intelligence   alpha = " %6.3f alphaI_CAS
display "Constant       alpha = " %6.3f alphaU
*
***

***DC-effects in Table 2.2 
* margin can be used for not observed combinations of X too using "noesample"
* predicted Probabilities and Difference of Probabilities


*** Table 2.2 Response probabilities of attending University;
*             simulated data set university
* note: command "margin" is used for computing probabilities
* favorable conditions
margin ,at(x4=(-21 0 21) x1=1 x2=1 x3=0 ) noesample
matrix tmp=r(b)
display "favorable conditions"
display "Probs p(y=1) if Country=B, Academic=yes and Sibling=no or one 
display "for intelligence I = ..."
display "I=-21: " tmp[1,1] ";  I=0: " tmp[1,2] ";  I=+21: " tmp[1,3]
display "difference I=0 - I=-21: DC = " %5.3f tmp[1,2]-tmp[1,1]
display "difference I=21 - I=0:  DC = " %5.3f tmp[1,3]-tmp[1,2]
* unfavorable conditions
margin ,at(x4=(-21 0 21) x1=0 x2=0 x3=1 ) noesample
matrix tmp=r(b)
display "favorable conditions"
display "Probs p(y=1) if Country=B, Academic=yes and Sibling=no or one 
display "for intelligence I = ..."
display "I=-21: " tmp[1,1] ";  I=0: " tmp[1,2] ";  I=+21: " tmp[1,3]
display "difference I=0 - I=-21: DC = " %5.3f tmp[1,2]-tmp[1,1]
display "difference I=21 - I=0:  DC = " %5.3f tmp[1,3]-tmp[1,2]
*
* note: options "pwcomp" of margin computes also DC-effects
margin ,at(x4=(-21 0 21) x1=1 x2=1 x3=0 ) pwcomp noesample
margin ,at(x4=(-21 0 21) x1=0 x2=0 x3=1 ) noesample pwcomp
*
***





 
*** chapter 2.2: Discrete response model -- DRM
*

** ch 2.2.1 Logistic regression, response profiles, Discrete (DC), 
* and Instantaneous (IC) Change measures

* ML-estimation of logistic regression of Y on X1, X2, X3, and X4 
*               with (co-)variances of betas
logit y x1-x4 [fw=f] ,cformat(%6.3f)
estat vce
* note: Diagonal elements are squares of standard errors 
*

** IC-effects AME and MEM
* note: though the predictors are discrete, for this example they are assumed
* to be continuous
* computing predicted probabilities of last (logit) model for AME & MEM : 
predict pL
label var pL "proabilities, estimated by logistic regression"
* mean of estimated probabilities to compute conditional error variances
sum pL [fw=f]
g s2error=pL*(1-pL) if e(sample)
sum s2error [fw=f]
* AME = product of regression coef. multiplied with average cond. error variance
display "AME of Country: " _b[x1]*r(mean)
display "AME of Academic: "_b[x2]*r(mean)
display "AME of Sibling "_b[x3]*r(mean)
display "AME of Intelligence: "_b[x4]*r(mean)
* direct estimation using Stata-command margin with option “dydx”
margin , dydx(x1 x2 x3 x4)
*
* MEM using margin-command
* probability at the means of X1 to X4
margin , dydx(x1 x2 x3 x4) atmeans
*

*
* Logit regression for university data with all possible interactions
* “c.varname” is used to declare a variable as metric-continuous;
* “varname1#varname2 computes product varname1*varname2 as additional predictor,
* “varname1##varname2##...#varnamek computes all lower order and main effects also
logit y c.x1##c.x2##c.x3##c.x4 [fw=f] ,cformat(%6.3f)
* note: all interactions are small and not significant
*
* LR-Test against main effect only model
estimates store h1
logit y x1-x4 [fw=f] , cformat(%6.3f) 
lrtest h1
estimates drop h1
*
**

** ch 2.2.2 Logistic DRM as a logit model: odds ratios as effect measures
* logits and odds-ratios:
logit y x1-x4 [fw=f] , cformat(%6.3f)
logit, or cformat(%6.3f)
* 
* Log. Odds, Odds, and probability of intercept (=reference group) for table 2.1 
display "Intercept betaU = log-odds of reference group: " %6.3f _b[_cons]
display "Intercept exp(betU) = odds(intercept) = " %5.3f exp(_b[_cons])
display "intercept probability = odds/(1+odds) = "
display " = " %6.4f exp(_b[_cons])/(1+exp(_b[_cons]))
*

* probability OR and logit change of increase in intelligence=0 to intelligence=8
*                          if Country=0 Academic=0 and Sibling=0:
margin ,at(x4=(0 8) x1=0 x2=0 x3=0)
margin ,at(x4=(0 8) x1=0 x2=0 x3=0) pwcomp 
* OR of this change: 
display "OR: " (0.1765217/(1-0.1765217)) / (0.0909112/(1-0.0909112))
* = exp(beta) powered by 8
display "= exp(beta) powered by 8 = " exp(_b[x4])^8
* logit change:
display "logit: " ln((0.1765217/(1-0.1765217)) / (0.0909112/(1-0.0909112)))
* = beta*8
display "= beta*8  " _b[x4]*8
*
* different change in probabilities but same change in OR and logits if country=1 academic=1 and sibling=1
* probabilities:
margin ,at(x4=(0 8) x1=1 x2=1 x3=1 )
margin ,at(x4=(0 8) x1=1 x2=1 x3=1 ) pwcomp
* OR:
display (0.6586128/(1-0.6586128)) / (0.4736874/(1-0.4736874))
display "= exp(beta) powered by 8 = " exp(_b[x4])^8
* logit:
display ln((0.6586128/(1-0.6586128)) / (0.4736874/(1-0.4736874)))
display "= beta*8  " _b[x4]*8
*
**

** ch 2.2.3 Probit regression

* Probit effects with their (co-) variances and probabilities
probit y x1-x4 [fw=f] , cformat(%6.3f) 
estat vce
predict pP
label var pP "proabilities, estimated by probit regression"
* compare probabilities of logit and probit regression for univerrsity data:
display "comparing logit and probit pobabilities:"
table x4 x1 x2 [fw=f] ,c(freq mean pL mean pP) by(x3)
*
* AME of probit regression
display "AME of probits"
margin , dydx(x1 x2 x3 x4)
*
* MEM of probit regression
display "MEM of probits"
margin , dydx(x1 x2 x3 x4) atmeans
*
* probit regression for university data with all possible interactions 
probit y c.x1##c.x2##c.x3##c.x4 [fw=f] ,cformat(%6.3f)
* Note: because generating model is logistic the probit curve has significant
*       interactions
*
*LR-Test against main effect only model
estimates store h1
probit y x1-x4 [fw=f] , cformat(%6.3f) 
lrtest h1
*
**


***  ch 2.2.4 Linear probability model – LPM
* PLM effects, (co-) variances and probabilities 
reg y x1-x4 [fw=f] , cformat(%6.3f)
estat vce
predict yhat
label var yhat "pedicted values = probabilities of Y=1"
* heteroscedastic-robust standard errors
reg y x1-x4 [fw=f] , vce(r) cformat(%6.3f)
estat vce 
* compare probabilities of logit and LPM linear regression for university data:
table x4 x1 x2 [fw=f] ,c(freq mean pL mean yhat) by(x3)
*
* probability differences
g diffP=yhat-pL
table x4 x2 x1 [fw=f] ,c(freq mean pL mean diffP) by(x3)
*
* AME = regrssion weights
margin , dydx(x1 x2 x3 x4)
*
* MEM = regression weights
margin , dydx(x1 x2 x3 x4) atmeans
*
*  PLM for university data with all possible interactions 
reg y c.x1##c.x2##c.x3##c.x4 [fw=f] ,cformat(%6.3f)
* Note: significant intedracfions
*
*LR-Test against main effect only model (assuming normal distributed residuals)
estimates store h1
reg y x1-x4 [fw=f] , cformat(%6.3f) 
lrtest h1
estimates drop h1
*
**


*** chapter 2.3: Latent variable model – LVM

** ch 2.3.1 Logistic latent variable model

* full and half underlying standardized effect in LVM-interpretation
* of logistic regression
*
* SD of Y* , note (n-1) weighted (by Stata)
logit y x1-x4 [fw=f]
* linear predictions 
predict xbL if e(sample), xb
sum xbL [fw=f]
* (n-1)-weighted variance of linear predictions 
scalar s2Ystar_hat=r(Var)
* variance of Ystar conditioned to error variance = pi²/3
scalar s2Ystar=r(Var)+_pi^2/3
* standard deviation of Ystar
scalar sdYstar=sqrt(s2Ystar)
*
* standardized LVM-effect of Country
sum x1 [fw=f] if e(sample)
display "half standardized logit effect logit y = " %6.3f _b[x1]/sdYstar
display "full standardized logit effect = "_b[x1]*r(sd)/sdYstar
*
* standardized LVM-effect of Academic
sum x2 [fw=f] if e(sample)
display "half standardized logit effect academic= "_b[x2]/sdYstar
display "full standardized logit effect= "_b[x2]*r(sd)/sdYstar
*
* standardized LVM-effect of Sibling
sum x3 [fw=f] if e(sample)
display "half standardized logit effect sibling= "_b[x3]/sdYstar
display "full standardized logit effect= "_b[x3]*r(sd)/sdYstar
*
* standardized LVM-effect of Intelligence
sum x4 [fw=f] if e(sample)
display "half standardized logit effect intelligence= "_b[x4]/sdYstar
display "full standardized logit effect= "_b[x4]*r(sd)/sdYstar
*

* LVM-R-Square of Ystar
display "LVM logit R² (n-1 weighted s²(yhat)) : " s2Ystar_hat/s2Ystar
*
**


** ch 2.3.2 Probit latent variable model

* full and half underlying standardized effect in LVM-interpretation 
* of probit regression
*
* SD of Y* , note (n-1) weighted
probit y x1-x4 [fw=f] 
* linear predictions (probits)
predict xbP if e(sample), xb
sum xbP [fw=f]
* (n-1)-weighted variance of linear predictions 
scalar s2Ystar_hat=r(Var)
* variance of Ystar restricted to error variance =1
scalar s2Ystar=r(Var)+1	
* standard deviation of Ystar
scalar sdYstar=sqrt(s2Ystar)
*
* standardized LVM-effect of Country
sum x1 [fw=f] if e(sample)
display "half standardized probit effect logit y = " %6.3f _b[x1]/sdYstar
display "full standardized probit effect = "_b[x1]*r(sd)/sdYstar
*
* standardized LVM-effect of Academic
sum x2 [fw=f] if e(sample)
display "half standardized probit effect academic= "_b[x2]/sdYstar
display "full standardized probit effect= "_b[x2]*r(sd)/sdYstar
*
* standardized LVM-effect of Sibling
sum x3 [fw=f] if e(sample)
display "half standardized probit effect sibling= "_b[x3]/sdYstar
display "full standardized probit effect= "_b[x3]*r(sd)/sdYstar
*
* standardized LVM-effect of Intelligence
sum x4 [fw=f] if e(sample)
display "half standardized probit effect intelligence= "_b[x4]/sdYstar
display "full standardized probit effect= "_b[x4]*r(sd)/sdYstar
*
* LVM-R-Square of Ystar
display "LVM preobit R²: " s2Ystar_hat/s2Ystar
*
**

** ch 2.3.3 Heteroskedastic errors, unequal thresholds and biased effects

* example of heterogenous residual variances and different thresholds 

* preserve university data
preserve
*
*  example data Y=attend university, X=Gender
clear
input Y X 
0 0 
0 1 
1 0 
1 1 
end
label var Y "Attending university"
label var X "Gender (1=male)"
*
* a) homoskedastic LVM-residuals and phi=0.5
* residual standard deviation = 0.5* pi/Square-root(3):
display "SD U =" %6.4f .5*_pi/sqrt(3) 
g Ystar_hat=-1.5+1*X
g pY=exp(Ystar_hat/0.5)/(1+exp(Ystar_hat/0.5)) 
replace pY=1-pY      if Y==0
*
*frequency table and logistic regression
* note: using importance-weight ("iw")
*       because data are conditional probabilities instead of frequencies
table Y X [iw=pY] ,col row
logit Y X [iw=pY] ,cformat(%6.3f)
logit ,or cformat(%6.3f)
* because phi=0.5  is known for both males and females:
* bYX = betaYX*phi is unbiased with exeption of rescaling both betaYX and intercept
display  "bYX = " %6.3f _b[X]*0.5 
display  "bY  = " %6.3f _b[_cons]*0.5
*

*
* b) heteroskedastic LVM-residuals: phi=1 for femals, 0.5 for males
* residual standard deviation is unchanged for males but doubled for femals 
* = pi/Square-root(3):
display "SD U fem. only =" %6.4f _pi/sqrt(3)
display "SD U male only =" %6.4f .5*_pi/sqrt(3)
g pY_het=pY                            if X==1
replace pY_het=exp(-1.5)/(1+exp(-1.5)) if X==0 
replace pY_het=1-pY_het                if X==0 & Y==0
table Y X [iw=pY_het] ,col row
logit Y X [iw=pY_het] ,cformat(%6.3f)
logit ,or cformat(%6.3f)
* estimated effect of Gender is now smaller!
* Note that the ratios betaYX/bY do change:
*   from homoscedastic szenario: _b[Gender]/_b[cons] = 2/-3     = -0.667
*   to heteroscedastic szenario: _b[Gender]/_b[cons] = 0.5/-1.5 = -0.333

*
* different proportions of Gender -> different pooled residual variances 
display "sigma_U|male = " %6.4f sqrt(_pi^2/3)
display "sigma_U|fem = " %6.4f sqrt(0.5^2*_pi^2/3)
*
* a) female 50% male 50%:
display "sigma_U_pooled = " %6.4f sqrt(.5*_pi^2/3 + .5*0.5^2*_pi^2/3)
display "phi = " sqrt(.5*0.5^2*_pi^2/3 + .5*_pi^2/3)/(_pi/sqrt(3))
display "betaYX = " sqrt(.5*0.5^2*_pi^2/3 + .5*_pi^2/3)/(_pi/sqrt(3))*(2-1.5)
display "intercept: " sqrt(.5*0.5^2*_pi^2/3 + .5*_pi^2/3)/(_pi/sqrt(3))*(-1.5)
*
* b) female 75% male 25%:
display "sigma_U_pooled = " %6.4f sqrt(.25*0.5^2*_pi^2/3 + .75*_pi^2/3)
display "phi = " sqrt(.25*0.5^2*_pi^2/3 + 75*_pi^2/3)/(_pi/sqrt(3))
display "betaYX = " sqrt(.25*0.5^2*_pi^2/3 + .75*_pi^2/3)/(_pi/sqrt(3))*(2-1.5)
display "intercept: " sqrt(.25*0.5^2*_pi^2/3 + .75*_pi^2/3)/(_pi/sqrt(3))*(-1.5)
*
* c) femal2 25% male 75%:
display "sigma_U_pooled = " %6.4f sqrt(.75*0.5^2*_pi^2/3 + .25*_pi^2/3)
display "phi = " sqrt(.75*0.5^2*_pi^2/3 + .25*_pi^2/3)/(_pi/sqrt(3))
display "betaYX = " sqrt(.75*0.5^2*_pi^2/3 + .25*_pi^2/3)/(_pi/sqrt(3))*(2-1.5)
display "intercept: " sqrt(.75*0.5^2*_pi^2/3 + .25*_pi^2/3)/(_pi/sqrt(3))*(-1.5)
* Note: the ratios betaYX/betaY0=-0.333 do not change by different proportions
*

*
* Different thresholds: 0 for femals, -2 for males 
*            and homoskedastic residual variance phi=0.5
* no change for females but (Y*hat-treshold)/phi for males
*     -> pY|fem = exp(-1.5/0.5)/(1+exp(-1.5/0.5))  
*        pY|male = exp((-1.5-(-2)+1)/0.5)/(1+exp((-1.5-(-2)+1)/0.5))
g pY_trh=pY                                      if X==0
replace pY_trh=exp(1.5/0.5)/(1+exp(1.5/0.5))     if X==1
replace pY_trh=1-pY_trh                          if X==1 & Y==0
table Y X [iw=pY_trh] , col row
logit Y X [iw=pY_trh] , cformat(%6.3f)
logit ,or cformat(%6.3f)
* Note: ratios betaYX/betaY0 changes from 2/-3 to  to 6/-3=2.0
*       caused by different thresholds only
*

* restore university data
restore
*
**
*** Chapter 2.4: Inserting mavericks, ‘orthogonal’ independent variables, 
*                into equations

* Correlations of predictors in university example
corr x1 x2 x3 x4 [fw=f]
* Reduced model: Attend univerity on country, academic & sibling 
logit y x1 x2 x3 [fw=f] , cformat(%6.3f)
est store Mr
* Full model: adding intelligencea as orthogonal independent variable 
logit y x1 x2 x3 x4  [fw=f] ,cformat(%6.3f)
est store Mf

*
* DC effects of country in full model
est restore Mf
logit
margin ,at(x1=(0 1) x2=1 x3=0 x4=0) pwcomp noesample
*
*DC efffects of ountry in restricted  model
est restore Mr
logit
margin ,at(x1=(0 1) x2=1 x3=0) pwcomp noesample
*
* -> DC effects smaller in restricted model
*

* AME in full model
est restore Mf
margin , dydx(x1 x2 x3 x4)
*
* AME in restricted model
est restore Mr
margin , dydx(x1 x2 x3)
* ->AME only slightly smaller in restricted model
*

log close _all
