log using chapter3.log , replace

******* Chapter 3 *******

* Simulated Data set University as in chapter 2
import excel "university.xlsx", sheet("data") firstrow clear
label var y "U: attending university"
label define y 0 "not attending" 1 "attending"
label value y y 
label var x1  "C country"
label def x1 0 "A" 1 "B"
label val x1 x1
label var x2 "A: academic background of parents"
label def x2 0 "no" 1 "yes"
label val x2 x2
label var x3 "S: sibling"
label def x3 0 "no or one" 1 "two or more"
label val x3 x3
label var x4 "I: quintiles from N(0,15) intelligence scores"
label var f "frequencies (weights)"
* rename variable as used in chapter 3
rename  (y x1 - x4 f) (Y C A S I freq)
*

* Equation (3.1): Note intercept (internal name “_cons”) is last element in Stata output:
display "Eq. (3.1)
logit Y C A S I [fw=freq] ,or  cformat(%6.3f)
*
*

***** Ch. 3.1 Comparing effect within a single LVM equation *****

*
* Statistical significance of difference of two regression weights:
* Examples: 
* a) Is weight of Country significant greater than weight of Academic?       
lincom C-A 
*          -> lincom computesd Z-test for the difference of regr. coefficients:
*
* b) is maximum effect of I > effect of C?
lincom 42*I-C
*

* full and half standardized effect in LVM-interpretation of logistic regression
*
logit
predict xbL if e(sample), xb
qui: sum xbL [fw=freq]
scalar s2Ystar_hat=r(Var)
scalar s2Ystar=r(Var)+_pi^2/3
scalar sdYstar=sqrt(s2Ystar)
qui: sum C [fw=freq] if e(sample)
scalar b_C_hs=_b[C]/sdYstar
scalar b_C_fs=_b[C]*r(sd)/sdYstar
qui: sum A [fw=freq] if e(sample)
scalar b_A_hs=_b[A]/sdYstar
scalar b_A_fs=_b[A]*r(sd)/sdYstar
qui: sum S [fw=freq] if e(sample)
scalar b_S_hs=_b[S]/sdYstar
scalar b_S_fs=_b[S]*r(sd)/sdYstar
qui: sum I [fw=freq] if e(sample)
scalar b_I_hs=_b[I]/sdYstar
scalar b_I_fs=_b[I]*r(sd)/sdYstar
scalar b_const_hs=_b[_cons]/sdYstar

* equation 3.2:
display "Eq. (3.2) full standardized LVM-regression:"  
display "Z_Y* = " b_C_fs "*Z_C + " b_A_fs "*Z_A + "  b_S_fs "*Z_S + " b_I_fs "*Z_I"
* equation 3.3
display "Eq. (3.3) half standardized LVM-regression:"  
display "Z_Y* = "  b_const_hs "*intercept +" b_C_hs "*C + " b_A_hs "*A + " b_S_hs "*S + "  b_I_hs "*I"
* LVM-R-Square of Ystar
display "LVM R²: " %6.4f s2Ystar_hat/s2Ystar
display "LVM R: " %6.4f sqrt(s2Ystar_hat/s2Ystar)
display "McFaddens Pseudo R² based on LR-statistics:" %6.4f e(r2_p)
*
**

***** Ch 3.2 Comparing effects within a single DRM equation *****

* Full model
logit , cformat(%6.3f)
logit , or cformat(%6.3f)
*

* restricted model without mavericks Country and Intelligence
logit Y A S  [fw=freq] ,cformat(%6.3f)
logit, or cformat(%6.3f)
*
* restricted model with Country Academic and Sibling
logit Y C A S  [fw=freq] ,cformat(%6.3f)
logit, or cformat(%6.3f)
*
* restricted model with Academic Sibling  and Intelligence
logit Y A S I [fw=freq] ,cformat(%6.3f)
logit, or cformat(%6.3f)
*
**

***** 3.3 Causal interpretations in LVM and DRM logistic regression *****

** Ch 3.3.2 Causal DRM; graphs, SEMs, and Pearl’s structural modeling approach

*
* Simulated data for Figure 3.1
* a) Path model with metrix Y following Figur 3.1
* S is exogenous variable
* X=0.7*S +Ex
* T=0.6*S +Es
* V=0.5*T +Ev1 
* model 0: Y0=1        +0.4*V    +E
* model 1: Y1=1 +0.4*X +0.4*V    +E
* model 2: Y2=1 +0.4*X +0.4*V +Q +E
* b) same path model with dichotomous dependent Ybin1 to Ybin3, 
* where 
* model 1: Prob(Ybin11=1) = exp(1+0.4*V)/(1+exp(1+04*V))
* model 2: Prob(Ybin12=1) = exp(1+0.4*X+0.4*V)/(1+exp(1+0.4*X+04*V))
* model 3: Prob(Ybin13=1) = exp(1+0.4*X+0.4*V+Q)/(1+exp(1+0.4*X+04*+QV))
* S, X, T, V, Q and E are standard normaL istributed
* note: Q is a maverick, E is the residual variable in standard regressions
* n=5000
*
* data generation
clear
set obs 5000
* generating 6 indpendent N(0/1) variables  Q S X T V E 
set seed 987654321
*
g Q=rnormal()
sum Q
replace Q=(Q-r(mean))/r(sd)
*
g S=rnormal()
reg S Q 
predict r, r
sum r
replace S=(r-r(mean))/r(sd)
drop r
*
g X=rnormal()
reg X S Q 
predict r, r
sum r
replace X=(r-r(mean))/r(sd)
drop r
*
g T=rnormal()
reg T X S Q 
predict r, r
sum r
replace T=(r-r(mean))/r(sd)
drop r
*
g V=rnormal()
reg V T X S Q
predict r, r
sum r
replace V=(r-r(mean))/r(sd)
drop r
*
g E=rnormal()
reg E V T X S Q
predict r, r
sum r
replace E=(r-r(mean))/r(sd)
drop r
*
* generating model equations between dependent variables 
replace X=0.7*S+sqrt(0.51)*X
replace T=0.6*S+0.8*T
replace V=0.5*T+sqrt(.75)*V
corr Q S T V X E ,  m c
*
* metric path models
g Y0=1      +0.4*V+  E
g Y1=1+0.4*X+0.4*V  +E
g Y2=1+0.4*X+0.4*V+Q+E
*
* generation of dichotomous Y:
* step 1: computation of logit probabilities based on Yhat of Y
* step 2: generation of dichotom. Ybin by binomial random realizations of step 1 
g p0=exp(1+0.4*V)/(1+exp(1+0.4*V))
g Ybin0=rbinomial(1,p0)
g p1=exp(1+0.4*X+0.4*V)/(1+exp(1+0.4*X+0.4*V))
g Ybin1=rbinomial(1,p1)
g p2=exp(1+0.4*X+0.4*V+Q)/(1+exp(1+0.4*X+0.4*V+Q))
g Ybin2=rbinomial(1,p2)
*
* 
* Model 0: no causal effect of X on Y0
*
* linear regression of metric depend variable Y0 on Y
reg Y0 X , cformat(%6.4f)
* note: significant linear effect of X is a spurious causal effect
*
*  X -> Y0 controlled by S T U and/or V
reg Y0 X S , cformat(%6.4f)
reg Y0 X T , cformat(%6.4f)
reg Y0 X V , cformat(%6.4f)
reg Y0 X S T , cformat(%6.4f)
reg Y0 X S V , cformat(%6.4f)
reg Y0 X T V , cformat(%6.4f)
reg Y0 X S T V , cformat(%6.4f)
* no effect of X on Y after adding S T and/or V
* 
* same logistic regression for dependent variable Ybin0
* bivarate logistik X -> Ynin0
logit Ybin0 X , cformat(%6.4f)
* note: significant logistic effect of X is a spurious causal effect
*
* logistic X -> Ybin0 controlled by S T U and/or V
logit Ybin0 X S , cformat(%6.4f)
logit Ybin0 X T , cformat(%6.4f)
logit Ybin0 X V , cformat(%6.4f)
logit Ybin0 X S T , cformat(%6.4f)
logit Ybin0 X S V , cformat(%6.4f)
logit Ybin0 X T V , cformat(%6.4f)
logit Ybin0 X S T V , cformat(%6.4f)
* note: effect of X is not significant if S T and/or V are predictors
*
*
* Model 1: direct causal effects of X and V on Y1
*
* bivariate linear Effect of X on Y1
reg Y1 X , cformat(%6.4f)
* note: significant causal effect is confounded 
*
*  X -> Y1 controlled by S T U and/or V
reg Y1 X S , cformat(%6.4f)
reg Y1 X T , cformat(%6.4f)
reg Y1 X V , cformat(%6.4f)
reg Y1 X S T , cformat(%6.4f)
reg Y1 X S V , cformat(%6.4f)
reg Y1 X T V , cformat(%6.4f)
reg Y1 X S T V , cformat(%6.4f)
* note: causal effect of X on Y1 is correct estimated
*
* same logistic regression for dependent variable Ybin1
* bivariate logistic X -> Ynin0
logit Ybin1 X , cformat(%6.4f)
* note: significant logistic effect of X is confounded by confounder V
*
* logistic X -> Ybin1 controlled by S T U and/or V
logit Ybin1 X S , cformat(%6.4f)
logit Ybin1 X T , cformat(%6.4f)
logit Ybin1 X V , cformat(%6.4f)
* S, T or V control confounder S with different regression weights 
* Effect of X becomes larger, the higher the effect of V, T or S is on Ybin1
logit Ybin1 X S T , cformat(%6.4f)
logit Ybin1 X S V , cformat(%6.4f)
logit Ybin1 X T V , cformat(%6.4f)
logit Ybin1 X S T V , cformat(%6.4f)
* note: S is(correctly) not significant if T or V is used in the regression
* note: S and T (correctly) aren't significant if in the model
* note: X and V are significant if in the model
*
* Model 2 = specified as Model 1 though Y2 has additional effect by Maverik Q 
*
* bivariate linear Effect of X on Y2
reg Y2 X , cformat(%6.4f)
* note: ignoring maverick Q don't change effect of X but R² becomes smaller
reg Y2 X Q  , cformat(%6.4f)
* note: controlling Q don't change effect of X but R² becomes larger
*
* model Y2 on X and V (and Q)
reg Y2 X V, cformat(%6.4f)
reg Y2 X V Q , cformat(%6.4f)
* note: correct estimations of causal effects
*
* model Y2 on X, V, S and T (and Q)
reg Y2 X S T V , cformat(%6.4f)
reg Y2 X S T V Q , cformat(%6.4f)
* note: correct estimations of causal effects
* => Mavericks can be omitted in estimation of linear causal effects
*
* same logistic regression for dependent variable Ybin2
* Model 2 = specified as Model 1 though Ybin2 has additional effect by Maverik Q 
*
* bivariate logistic Effect of X on Ybin2
logit Ybin2 X , cformat(%6.4f)
* note: effect of X becomes  smaller than with dependent Ybin1
logit Ybin2 X Q  , cformat(%6.4f)
* note: effect of X becomes larger, but effect of Q is smaller because
*       V is omitted and V is a Maverick with respect to Q
*
* model Y2 on X and V (and Q)
logit Ybin2 X V, cformat(%6.4f)
logit Ybin2 X V Q , cformat(%6.4f)
* note: effects of X and V smaller if Maverick Q is omitted
*
* model Y2 on X, V, S and T (and Q)
logit Ybin2 X S T V , cformat(%6.4f)
logit Ybin2 X S T V Q , cformat(%6.4f)
* note: correctly X and V are significant and S and T not
*      effects of and V smaller if Maverick Q is omitted 
**

** ch 3.3.3 Causal DRM; Rubin’s Potential Outcome approach

*
* Import data
* data from Stavrova, Newman, Kulemann, Fetchenhauer 2016, pp. 562-63
import excel "moral.xlsx", sheet("table1") firstrow clear
label var Y "Response "
label def Y 1"same " 0"other"
label val Y Y
label var X "Moral"
label def X 0"immoral" 1"moral"
label val X X
*
* expand to individual cases
expand n
drop n
*
tab2 Y X  ,col
logit Y X  ,cformat(%6.4f)
logit ,or cformat(%6.4f)
predict Pr_mean , pr
g p_y1x1=59/92
g p_y1x0=28/100 

* 
*
* adding individual uniform distributed random noise to each case
set seed 123456789
* noise for X=1
g noise_x1=runiform(-0.1,0.1)
by X, sort: egen tmp=mean(noise_x1)
replace noise_x1=noise_x1-tmp
drop tmp
* noise for X=0
g noise_x0=runiform(-0.1,0.1)
by X, sort: egen tmp=mean(noise_x0)
replace noise_x0=noise_x0-tmp
drop tmp
*
*compute individual probabilities without noise for Y=1
* If X=1 (counterfactual if X=0):
g p_y1x1_i=p_y1x1+noise_x1
* If X=0 (counterfactual if X=1):
g p_y1x0_i=p_y1x0+noise_x0
* compute individual OR
* a) same ORs for all (without noise)
g OR=(p_y1x1/(1-p_y1x1)) / (p_y1x0/(1-p_y1x0))
* b) different ORs (with noise)
g OR_i=(p_y1x1_i/(1-p_y1x1_i)) / (p_y1x0_i/(1-p_y1x0_i))
*
* Results:(mean) OR with equal probs for all cases
*         < mean OR for different individual probs
sum noise_x1 noise_x0 p_y1x1 p_y1x1_i p_y1x0 p_y1x0_i OR OR_i
*
log close _all

