log using chapter4.log, replace

******* Chapter 4 *******

** Tenure data 
*
** Data for this example are stored on the hompepage of Scott S. Long 
*  in the file "https://jslsoc.sitehost.iu.edu/stata/spex_data/wf-tenure.dta"
*  note: cases are: individual times years 
*                   see Allison (1982) that these data discrete event histry data
*                   can be analyzed by logistic regression
* 
use  "wf-tenure.dta", clear
*
*  keep cases only used in Long et al. (1993), see also (Allison 1999)
keep if sampleis==1
drop sampleis
*
* rename variable from Long as used into names used in Hagenaars et al. (2024):
rename (tenure articles year yearsq) (y x v1 v1_sqr)
rename (prestige select female)( v2 v3 g)
*
label var y "reiceiving tenure 1=yes 0=no"
label var x "productivity (cumulated no. of articles)"
label var v1 "duration (years)"
label var v1_sqr "durarion squared"
label var v2 "prestige of department"
label var v3 "selectivity (baccalaureate)"
label var g "Gender 1=female 0=male"
*
*  computing interaction (product) variables with gender
g x_g=x*g
g v1_g=v1*g
g v1_sqr_g=v1_sqr*g
g v2_g=v2*g
g v3_g=v3*g
*
*

***** Chapter 4.1 Interaction effects in LVM 

* total group equation
* note: Interaction effects are estimated as effects of product variables 
logit y x v1 v1_sqr v2 v3 g x_g v1_g v1_sqr_g v2_g v3_g , cformat(%6.4f)
*
* separate group equation for males and females
bysort g: logit y x v1 v1_sqr v2 v3 , cformat(%6.4f) 
* note: (main) effect of x v1 v1_sqr v2 and v3 on y are equal in total and 
*       separate group analysis
* note: interaction effect (from product variables) are differences between 
*       variable effects in separate estimations
*       e.g. interaction: 
*            x times g =-0.0397 = x|g==1 - x|g==0 = -0.0340 - -0.0737 
*
* ratio of effect of x between males and females:
display "ratio beta_x|g==0/beta_x|g==1: " 0.0340/0.0737
*
*preserve data
preserve
*

* simulated example: effect of different thresholds or scaling factor 
*                     (phi )depending on variable g:
* simulated data: y*= -1.0 +1.0*x -0.5*g +0.5*x*g +u if g=0: phi=1, threshold=1
*                 y*= -1.0 +1.0*x -0.5*g +0.5*x*g +u if g=1: phi=1, threshold=-1
*                 n=2400
clear
set seed 987654321
set obs 2400
* generating independent variable x
g x=0        if _n<801
replace x=1  if _n<1601 & _n>800
replace x=2  if _n>1600
* generating moderator variable g
g g=0
replace g=1  if (_n>400&x==0) | (_n>1200&x==1) | (_n>2000&x==2)
* generating latent residual variable u
g u=rlogistic()
* mean(u)=0 in all subgroups defined by x times g
bysort x g: egen tmp=mean(u)
replace u=u-tmp
drop tmp
*
* Model 1: equal threshold and scaling factor:
* generating Y*
g y_star=-1.0+1*x-0.5*g+0.5*x*g+u
* linear Regression of y* on x
bysort g: reg y_star x 
* generating y by dichotomizing y* at threshold 1  
g y=(y_star >= 1)
bysort g: logit y x 
*

* Model 2: different thresholds
drop y
g y=(y_star>=1)        if g==0
replace y=(y_star>=-1) if g==1 
*
* separat group estimation
* g==0
logit y x if g==0
margin , predict(xb) at(x=(0 1 2))
* g==1
logit y x if g==1
margin , predict(xb) at(x=(0 1 2))
drop y y_star
*

*** effects of different scaling factor phi
* Model 1: y1*: equal phi=1
g y1_star=-1+1*x-0.5*g+0.5*x*g+u
* Model 2: y2*: phi=1 if g=0 & phi=2 if g==1
g y2_star=y1_star                            if g==0
replace y2_star=-1+1*x-0.5*g+0.5*x*g+2*u     if g==1
* Model 3: y3*: phi=1 if g=0 & phi=0.5 if g==1
g y3_star=y1_star                            if g==0
replace y3_star=-1+1*x-0.5*g+0.5*x*g+0.5*u   if g==1
*
* Dichotomization at threshold 0
g y1=(y1_star>=0)
g y2=(y2_star>=0)
g y3=(y3_star>=0)
*
* Model 1 (equal phi): separate linear regression of y1*
by g: reg y1_star x
* Model 1 (equal phi): separate logistic regression of y1 
by g: logit y1 x
*
* Model 2 (phi|g=1 > phi|g=0): separate linear regression of y2*
by g: reg y2_star x
* Model 2 (phi|g=1 > phi|g=0): separate logistic regression of y2
by g: logit y2 x
*
* Model 3 (phi|g=1 < phi|g=0): separate linear regression of y3*  
by g: reg y3_star x
* Model 3 (phi|g=1 < phi|g=0): separate logistic regression of y3
by g: logit y3 x
*
*
* total group analysis using linear regression or logistic regression 
* generating product variable x_g=x* g
g x_g=x*g
* Model 1: homoskedastic error variances
reg y1_star x g x_g
logit y1 x g x_g
* Model 2: residual variance larger if g=1
reg y2_star x g x_g
logit y2 x g x_g
* Modedl 3: residual variance smaller if g=1
reg y3_star x g x_g
logit y3 x g x_g
*

* restoring tenure data set
restore
*
* assuming effect of prestige on y* is equal for males and females 
* subgroup g=0
logit y x v1 v1_sqr v2 v3 if g==0
scalar v2male=_b[v2]
scalar xmale=_b[x]
logit y x v1 v1_sqr v2 v3 if g==1
scalar v2fem=_b[v2]
scalar xfem=_b[x]
* ratio of phi assuming equal effect in both groups
display "ratio of SD residuals = " v2male/v2fem
display "corrected ratio of productivity effect"
display "   = " (xmale/xfem) / (v2male/v2fem)
*
*

***** chapter 4.2 Interaction effects in DRM 

* separate group analysis:
bysort g: logit y x v1 v1_sqr v2 v3 , cformat(%6.4f) 
*
* direct estimation (weight of x_g) in total group equation
* note: opposite sign for gender & interactions effects because reference group
*       is male (g==0) 
*       -> interactions = effect males -effect females
logit y x v1 v1_sqr v2 v3 g x_g v1_g v1_sqr_g v2_g v3_g , cformat(%6.4f)
*
* Odds Ratios:
logit , or cformat(%6.4f)
display "OR-Interaction between Productivity and Gender:"
display "   female effect / male effect: "
display "   exp(-_b[x_g])  = " exp(0.0397)
* 

preserve
* effect of mavericks (variable mav) in logistic regression with interaction
* effects 
* simulated data: y*=-1.0 +1.0*x -0.5*g+0.5*g*x+mav+u ; mav=N(0;1) 
*                 y*=-1.0 +1.0*x -0.5*g+0.5*g*x+mav+u ; mav=N(0;1|g=0)
*                                                     & mav=N(0,.25|g=1) 
*                 n=2400
clear
set seed 987654321
set obs 2400
g x=0        if _n<801
replace x=1  if _n<1601 & _n>800
replace x=2  if _n>1600
g g=0
replace g=1  if (_n>400&x==0) | (_n>1200&x==1) | (_n>2000&x==2)
g u=rlogistic()
bysort x g: egen tmp=mean(u)
replace u=u-tmp
drop tmp
g xg=x*g
*generating maverick
g tmp=rnormal()
qui: reg tmp u
predict mav ,r
drop tmp
bysort x g: egen tmp1=mean(mav)
bysort x g: egen tmp2=sd(mav)
replace mav=(mav-tmp1)/tmp2
drop tmp1 tmp2
*
* Model 1: without maverick y1star=-1+1*x-0.5*g+0.5*x*g+u
g y1star=-1+x-.5*g+.5*xg+u
g y1=(y1star>=0)
reg y1star x g xg
logit y1 x g xg
*
* Model 2: with maverick equal distributed in both groups
g y2star=-1+x-.5*g+.5*xg+mav+u
g y2=(y2star>0)
corr x g xg mav
* nested regression: restricted model maverick not specified
*                    full model maveric specified as further independent variable
nestreg: reg y2star (x g xg) (mav) 
nestreg, lr: logit y2 (x g xg) (mav) 
*
* Model 3: maverick with different variances of maverics given gender 
g y3star=-1+x-.5*g+.5*xg+mav+u           if g==0
replace y3star=-1+x-.5*g+.5*xg+0.5*mav+u if g==1
* generating interaction g*mav
g g_mav=g*mav
g y3=(y3star>=0)
*nested linear regression on y3*
nestreg: reg y3star (x g xg) (mav) (g_mav)
* nestef logistic regression on y3
nestreg, lr: logit y3 (x g xg) (mav) (g_mav)
*
*

***** chapter 4.3 Interaction and causal analysis 
*

* example of dependency of interaction from statistical model to analyse the data
* simulated data for Income (i) on Education (e) moderated by gender (g)
* defining data:
clear
input i e g n1 n2 n3 n4
1 1 1 600 750 600 950
0 1 1 400 250 400  50
1 0 1 450 500 400 850
0 0 1 550 500 600 150
1 1 0 250 250 150 600
0 1 0 750 750 850 400
1 0 0 100 100  50 400
0 0 0 900 900 950 600
end
label var i "income 1=high 0=low"
label var e "education 1=high 0=low"
label var g "gender 1=males 0=females"
label var n1 "cases model 1"
label var n2 "cases model 2"
label var n3 "cases model 3"
label var n4 "cases model 4"
*
* computing product variable
g e_g=e*g
*
*
* Model 1: linear regression without interaction, logistic regression
*          with negative interaction e*g
table i e [fw=n1] , by(g)
nestreg: reg i (e g) (e_g) [fw=n1] , cformat(%6.4f)
nestreg, lr: logit i (e g) (e_g) [fw=n1] , or cformat(%6.4f)
*
* Model 2: linear regression with positive interaction e*g, logistic regression
*          without interaction
table i e [fw=n2] , by(g)
nestreg: reg i (e g) (e_g) [fw=n2] , cformat(%6.4f)
nestreg, lr: logit i (e g) (e_g) [fw=n2] , or cformat(%6.4f)
*
* Model 3: linear regression with positive interaction e*g, logistic regression
*           with negative interaction e*g
table i e [fw=n3] , by(g)
nestreg: reg i (e g) (e_g) [fw=n3] , cformat(%6.4f)
nestreg, lr: logit i (e g) (e_g) [fw=n3] ,or cformat(%6.4f)
*
* Model 4: linear regression with negative interaction e*g, logistic regression
*           with positive interaction e*g
table i e [fw=n4] ,by(g)
nestreg: reg i (e g) (e_g) [fw=n4] , cformat(%6.4f)
nestreg, lr: logit i (e g) (e_g) [fw=n4] ,or cformat(%6.4f)
*

*
log close _all
