log using chapter5.log, replace

******* Chapter 5: Tables *******

***** Preparing discrimination data (Hunter and Walker 1996)
*
* aggregated data set "discrimination data.xlsx" from Hunter and Walker (1996)
* data are stored as multidimensional frequency table in the
* EXCEL-sheet in columns  A: M minorities; 1=black, 0=non-black)
*                         B: P credit history:
*                            >50 days dept past due; 1=yes, 0=no
*                         C: D mortgage loan applicaion denied; 1=yes, 0=no
*                         D: f frquencies)
*
* import data
import excel "discrimination data.xlsx", sheet("table5.1a") firstrow clear
*
label var M "minorities"
label def M  0"othwerwiase" 1"black" 2"otherwiswe"
label val M M
*
label var P "credit history"
label def P  0"otherwise" 1"past due" 2"otherwise"
label val P P
*
label var D "mortgage loan application denied; 1=yes, 0=no" 
label def D 0"otherwise" 1"denied" 2"otherwisae"
label val D D
*
label var f "frequencies (weights)"
*
* generating MxP
* col =: combination of Minority (M) and Credit History (P)
recode M P D (0=2)
egen col=group(M P) 
lab var col "M x P"
lab def col 1"1 1" 2"1 0" 3"0 1" 4"0 0"
lab val col col
*

*** Table 5.1 Discrimination example: Observed data (Hunter and Walker 199)
*
recode M P D (0=2)
*
** 5.1a: table M P D, observed frequencies 
table col D ,c(sum f) col row
*
** 5.1b: table M P D, observed row percentages
tab2 col D [fw=f] ,row
*
** 5.1c: table M D , observed row percentages 
tab2 M D [fw=f],row
*
** 5.1d: table M P, observed row percentages
tab2 M P [fw=f],row
*
*  recode M, P, and D to original values
recode M P D (2=0)
*
***

*** Table 5.2 Discrimination example: Estimated data for main-effects-only model 
*
** Preparing data
recode M P D (2=0)
* computation of cond. probabilities D|MP using logistic regression
logit D M P [fw=f]
predict pD_MP ,p
replace pD_MP=1-pD_MP if D==0
* pD_MP: predicted estimated cond. probabilities of D|MP based on linear logits
* save frequencies of MxP as variable fDxM in discriminattion data file
preserve
table col ,c(sum f) replace
rename table1 fDxM
save tmp, replace
restore
merge n:1 col using tmp
drop _merge
* compute expected frequencies of pD|MP rounded after multiplication with 1000
g eD_MP=fDxM*pD_MP
label var eD_MP "expected freq. based on logits of D|MP"
g tmp=round(eD_MP*1000)
*
** 5.2a: table M P D estimated frequencies EQ(5.10b)
recode D M P (0=2)
*
table col D ,c(sum eD_MP) f(%8.3f) col row
*
** 5.2b: estimated row percentages based on eD_MP*1000
tab2 col D [fw=tmp] ,row 
*
** 5.2c: table MD, collapsing Table 5.2ab (same as Table 5.1c)
tab2 M D [fw=tmp] ,row 
*
drop tmp
*
***

*** Table 5.3 Basic effect estimates in Discrimination example 
*             (based on data of table 5.2a)
*
* computations
recode M P D (2=0) 
*
* model M -> D <- P , (D*-stand.-) , betas and var D* saved in sclars 
logit  D M P [iw=eD_MP]
scalar betaDM_P=_b[M]
scalar betaDP_M=_b[P]
predict tmp ,xb
sum tmp [iw=eD_MP]
scalar SDyDMP=sqrt(r(Var)+_pi^2/3)
scalar betaDM_Phs=_b[M]/SDyDMP
scalar betaDP_Mhs=_b[P]/SDyDMP
drop tmp
*
* model M -> D <- residualP_M = rP_M , results saved in scalars
*  standard regression of P on M to save residualas
reg P M [fw=f]
predict rP_M ,r
label var rP_M "residual of OLS P|M"
*  logistic regression
logit D M rP_M [iw=eD_MP]
scalar betaDM_rP=_b[M]
scalar betaDM_rPhs=_b[M]/SDyDMP
*
* model M -> D , results saved in scalars
logit D M [fw=f]
scalar betaDM=_b[M]
predict tmp ,xb
sum tmp [fw=f]
scalar SDyDM=sqrt(r(Var)+_pi^2/3)
scalar betaDMhs=_b[M]/sqrt(r(Var)+_pi^2/3)
drop tmp
*
* model M -> P both OLS & logit , results saved in scalars
reg P M [fw=f]
scalar bPMols=_b[M]
logit P M [fw=f]
scalar betaPM=_b[M]
*
** Table 5.3 Basic effect estimates in Discrimination example
*
* a) effects of M on D/D*:"
display "bivariate logistic: betaDM_hat = " %6.4f betaDM
display "halfstandardized D* on M: bDM_hat_hs = " %6.4f betaDMhs
display "partial logistic: betaDM.P_hat = " %6.4f betaDM_P
display "halfstandardized D* on M|P : bDM.P_hat_hs = " %6.4f betaDM_Phs
display "controlled for Residuals of P|M: betaDM.rP_hat = " %6.4f betaDM_rP
display "halfstandardized for D*: bDM.rP_hat_hs = " %6.4f betaDM_rPhs
* b) effects of P on D/D*
display "effects of P on D/D*:"
display "partial logistic: betaDP.M_hat = " %6.4f betaDP_M
display "halfstandardized D* on P|D : bDPM.P_hat_hs = " %6.4f betaDP_Mhs
* c) bivariate effects of M on P
display "effects of M on P:"
display "bivariate linear (LPM): bPM_hat = " %6.4f bPMols
display "bivariate logistic: betaPM_hat = " %6.4f betaPM
*
***

*** Table 5.4 LVM indirect Effect estimates: M->P->->D*; Discrimination example 
*
** a) naive procedure with respect to logistic regression
display "naive procedure ignoring scaling & incompatibilities"
display "size indirect effect:" 
display "betaDM_hat - betaDM.P_hat = " %6.4f betaDM-betaDM_P
display "relative indirect effect:"
display "1-betaDM.P_hat/betaDM_hat = 1 - " %6.4f betaDM_P/betaDM
display "                              = " %6.4f 1-betaDM_P/betaDM
*
** b) D*-standardized effects:
display "D*-standardized effects:"
*
display "scale corrected (only)"
display "size indirect effect:" 
display "betaDM_hat_hs - betaDM.P_hat_hs = " %6.4f betaDMhs-betaDM_Phs
display "relative indirect effect: 1-bDM.P_hat_hs/bDM_hat_hs"
display "  1 - " %6.4f betaDM_Phs/betaDMhs " = " %6.4f 1-betaDM_Phs/betaDMhs
*
display "scale + incompatibility corrected "
display "size indirect effect: "
display "bPMhat * bDP.M_hat_hs = " %6.4f bPMols*betaDP_Mhs
display "relative indirect effect:"
display "  1 - bDM.P_hat_hs/(bDM.p_hat_hs+bPMhat * bDP.M_hat_hs)"
display "  1 - " %6.4f betaDM_Phs/(betaDM_Phs+bPMols*betaDP_Mhs)
display " = " %6.4f 1-betaDM_Phs/(betaDM_Phs+bPMols*betaDP_Mhs)
*
** c) KHB
display "size indirect effect: "
display "betaDM.rP_hat_hs- betaDM.P_hat_hs = " %6.4f betaDM_rPhs-betaDM_Phs
display "relative indirect effect: 1-bDM.P_hat_hs/bD.rPM_hat_hs"
display "1 - " %6.4f betaDM_P/betaDM_rP " = " %6.4f 1- betaDM_P/betaDM_rP
*
** d) path analytic 
display "path analytic method: relative indirect effect:"
display "  1 - betaDM.P_hat/(betaDM.P_hat+bPMhat * bDP.M_hat"
display "  1 - " %6.4f betaDM_P/(betaDM_P+bPMols*betaDP_M)
display "      = " %6.4f 1-betaDM_P/(betaDM_P+bPMols*betaDP_M)
*
***

*** Table 5.5 DRM indirect Effect estimates: M->P->D*; Discrimination example 
*
** SIMMAV-computation
*      marginal (univariate) probabilities of P
recode M P D (2=0)
logit P [fw=f]
predict pP
replace pP=1-pP if P==0
*      marginal (univariate) probabilities of M
logit M [fw=f]
predict pM
replace pM=1-pM if M==0
*       counterfactual Probs = Prob(D|MP)*prob(P)*prob(M)
g pD_MP_mav = pD_MP*pP*pM
lab var pD_MP_mav "counterfactual SIMMAV probabilities"
*       compute counterfactual SMMAV frequencies (expected obs)
sum f
scalar n_obs=r(N)*r(mean)
g nD_MP_mav=pD_MP_mav*n_obs
lab var nD_MP_mav "counterfactual SIMMAV frequencies"
* (direct) logit SIMMAV-effect in betaDM_hat_mav
logit D M  [iw=nD_MP_mav] 
scalar betaDMmav=_b[M]
* DC SIMMAV-effect in bDMmav_direct
reg  D M [iw=nD_MP_mav]
scalar bDMmav_direct=_b[M]
*
** SIMNAT computations
*   Non-Blacks have counterfactual same P|M-probs as Blacks
*   Blacks with their natural P|M-distribution
* note: order of variables and categories as in Table 5.1
* counterfactual freqs for direct effect innMPD_M1all
logit P M [fw=f]
predict pP_M ,p
replace pP_M=1-pP_M if P==0
label var pP_M "prob(P|M)"
g pP_M1all=pP_M if M==1
replace pP_M1all=pP_M[_n-4] if M==0
lab var pP_M1all "counterfact. prob(P|M) where M=0 as M=1"
g nMPD_M1all=pD_MP*pP_M1all*pM*n_obs
label var nMPD_M1all "counterfact. freq for M=0 as M=1 in Pr(P|M)"
logit D M [iw=nMPD_M1all]
scalar betaDM_dnatM1=_b[M]
*   Blacks have counterfactual same P|M-probs as Non-Blacks
*   Non blacks with their natural P_M-distribution
* counterfactual freqs for direct effect innMPD_M0all
g pP_M0all=pP_M if M==0
replace pP_M0all=pP_M[_n+4] if M==1
lab var pP_M0all "counterfact. prob(P|M) where M=1 as M=0 "
g nMPD_M0all=pD_MP*pP_M0all*pM*n_obs
label var nMPD_M0all "counterfact. freq for M=1 as M=0 in Pr(P|M)"
logit D M [iw=nMPD_M0all]
scalar betaDM_dnatM0=_b[M]
*
*Table 5.5.:
** a) naive procedure with respect to logistic regression
display "naive procedure"
display "size indirect effect:" 
display "betaDM_hat - betaDM.P_hat = " %6.4f betaDM-betaDM_P
display "relative indirect effect:"
display "1-betaDM.P_hat/betaDM_hat = 1 - " %6.4f betaDM_P/betaDM
display "                              = " %6.4f 1-betaDM_P/betaDM
*
** b) path analytic / KHB
display "path analytic  / KHB "
display "size indirect effect: "
display "bPMols*betaDP.Mhat = " %6.4f bPMols*betaDP_M
display "relative indirect effect:"
display "1-betaDM.P_hat/(betaDM_Phat + bPMols*betaDP.Mhat):"
display " 1-  " %6.4f betaDM_P/(betaDM_P+bPMols*betaDP_M)
display "    = " %6.4f 1-betaDM_P/(betaDM_P+bPMols*betaDP_M)
* collapsing effect (CL):
display "difference CL-effect:"
display "betaDM_hat - betaDM.rP_hat = " %6.4f betaDM-betaDM_rP
display "difference CL-effect: betaDM.rP_hat/betaDM_hat"
display "                    = " %6.4f betaDM_rP/betaDM
*
** c) SIMMAV
display "SIMMAV counterfactual data assuming D is a maverick for P"
display "direct SIMMAV effect of M on D: betaDM_mav = " %6.4f betaDMmav
display "size indirect effect: "
display "betaDM_hat - beta_DM_mav = " %6.4f betaDM-betaDMmav
display "relative indirect effect: 1 - beta_DM_mav/beta_DM_hat"
display "  1 - " %6.4f betaDMmav/betaDM " = " %6.4f 1-betaDMmav/betaDM
* collapsing effect:
display "difference CL-effect:"
display "betaDM_mav - betaDM.P_hat = " %6.4f betaDMmav-betaDM_P
display "relative CL-effect:"
display " betaDM.P_hat/betaDM_mav = " %6.4f betaDM_P/betaDMmav
*
** c) SIMNAT
display "SIMNAT J. Pearls natural effects using counterfactual data "
display "M=0 as counterfacuals:"
display "direct natural effect betaDM_dnat:" %6.4f  betaDM_dnatM1
display "size indirect effect: "
display "betaDM_hat - beta_DM_dnat = " %6.4f betaDM-betaDM_dnatM1
display "relative indirect effect:"
display "1-betaDM_dnat/betaDM_hat = " %6.4f 1-betaDM_dnatM1/betaDM
display "difference CL-effect:"
display "betaDM_dnat - betaDM.P_hat = " %6.4f betaDM_dnatM1-betaDM_P
display "relative CL-effect:"
display " betaDM.P_hat/betaDM_dnat = " %6.4f betaDM_P/betaDM_dnatM1
*
display "M=1 as counterfactuals:"
display "direct natural effect betaDM_dnat:" %6.4f  betaDM_dnatM0
display "size indirect effect: "
display "betaDM_hat - beta_DM_dnat = " %6.4f betaDM-betaDM_dnatM0
display "relative indirect effect:"
display "1-betaDM_dnat/betaDM_hat = " %6.4f  1-betaDM_dnatM0/betaDM
display "difference CL-effect:"
display "betaDM_dnat - betaDM.P_hat = " %6.4f betaDM_dnatM0-betaDM_P
display "relative CL-effect:"
display " betaDM.P_hat/betaDM_dnat = " %6.4f betaDM_P/betaDM_dnatM0
*
***

*** Table 5.6 SIMMAV – Simulating mavericks; Discrimination example
recode M P D (0=2)
*
*   5.6a SIMMAV frequencies MPD 
table col D ,c(sum nD_MP_mav) row col f(%8.3f)
*
*   5.6b SIMMAV percentages D|MP
g tmp =round(nD_MP_mav*1000)
tab2 col D [fw=tmp] , row
*
*   5.6c SIMMAV frequencies MD 
table M D ,c(sum nD_MP_mav) row col  f(%8.3f)
*
*   5.6d SIMMAV percentages D|M 
tab2 M D  [fw=tmp] , row  
drop tmp
recode M P D (2=0)
*
***

*** Table 5.7 SIMNAT – Simulating Natural (‘pure’) Effects;
*             Discrimination example
** computations 
recode M P D (2=0)
*indirect effects for counterfactual Non_blacks (M=0 as M=1)
g nMPD_M1ind=nMPD_M1all[_n+4]  if M==1
replace nMPD_M1ind=eD_MP if M==0
*indirect effects for counterfactual blacks (M=1 as M=0)
g nMPD_M0ind=nMPD_M0all[_n-4]  if M==0
replace nMPD_M0ind=eD_MP  if M==1
*
**  5.7a est. frequencies direct natural effect: (M=1 actual)
recode M P D (0=2)
table col D ,c(sum nMPD_M1all) row col f(%8.3f)
*
**  5.7b Table MD (est.perc. from collapsed Table 5.7a)  
g tmp =round(nMPD_M1all*1000)
tab2 M D [fw=tmp] , row 
drop tmp
*
**  5.7c est. frequencies indirect natural effect: (M=1 actual)
table col D ,c(sum nMPD_M1ind)  row col f(%8.3f)
*
**  5.7d Table MD (est.perc. from collapsed Table 5.7c)
g tmp =round(nMPD_M1ind*1000)
tab2 M D [fw=tmp] , row 
drop tmp
*
*
** same tables for counterfactual Blacks (M=1 as M=0)
display "M=1 as counterfactuals, tables not shown:"
*
** a) est. frequencies direct effect: (M=0 actual)
table col D ,c(sum nMPD_M0all) row col f(%8.3f)
*
** b) Table MD (est. perc. from collapsed Table a))  
g tmp =round(nMPD_M0all*1000)
tab2 M D [fw=tmp] , row 
drop tmp
*
** c) est. frequencies indirect effect: (M=1 actual)
table col D ,c(sum nMPD_M0ind)  row col f(%8.3f)
*
** d) Table MD (est. perc. from collapsed Table c)
g tmp =round(nMPD_M0ind*1000)
tab2 M D [fw=tmp] , row 
drop tmp
*
***

*** Table 5.8 SIMNAT: Total effect as the sum
*             of the natural (‘pure’) direct and natural indirect effect; 
*
** computations for Non-Blacks as Blacks (M=1 actual)
recode M P D (2=0)
reg D M [fw=f]
scalar bDMols=_b[M]
*
reg D M [iw=nMPD_M1all]
scalar bDMdnat_M1=_b[M]
*
logit D M  [iw=nMPD_M1ind]
scalar  betaDM_indM1=_b[M]
reg D M [iw=nMPD_M1ind]
scalar bDMind_M1=_b[M]
*
**  8a Table MD. obs. freq. Table 5.1c (=5.2c)
recode M P D (0=2)
table M D ,c(sum f)
display " OR_DM  " %6.4f exp(betaDM)
display "betaDM: " %6.4f betaDM
display " DC_DM: " %6.4f bDMols
*
** 8b Table MD (direct effect Table 5.7b)
table M D ,c(sum nMPD_M1all) f(%8.3f)
display " OR_DM  " %6.4f exp(betaDM_dnatM1)
display "betaDM: " %6.4f betaDM_dnatM1
display " DC_DM: " %6.4f bDMdnat_M1
*
** 8c Table MD (indirect effect Table 5.7d)
table M D ,c(sum nMPD_M1ind) f(%8.3f)
display " OR_DM  " %6.4f exp(betaDM_indM1)
display "betaDM: " %6.4f betaDM_indM1
display " DC_DM: " %6.4f bDMind_M1
*
* Effect decomposition of M on D using natural effects
display "effect decomposition of DM using natural effects"
display "OR_DM = OR_DM_Dnat*OR_DM_ind"
display %6.4f exp(betaDM)
display " = " %6.4f exp(betaDM_dnatM1) " * " %6.4f exp(betaDM_indM1)
*
display "betaDM = betaDM_Dnat + betaDM_ind"
display %6.4f betaDM " = " %6.4f betaDM_dnatM1 " + " %6.4f betaDM_indM1
*
display "DC_DM = DC_Dnat + DC_DM_ind"
display %6.4f bDMols " = " %6.4f bDMdnat_M1 " + " %6.4f bDMind_M1
*
***

*** Table 5.9 SIMMAV: Total effect as the sum of the modified
*             natural (‘pure’) direct and modified natural indirect effect; 
*
** computations for counterfactual indirect Maverick effect in Tab. 5.9 c+d
recode M P D (2=0)
g nMPDmav_indM1=eD_MP                  if M==1
replace nMPDmav_indM1=nD_MP_mav[_n-4]  if M==0
logit D M [iw=nMPDmav_indM1]
scalar betaDMmav_indM1=_b[M]
reg D M [iw=nMPDmav_indM1]
scalar bDMmav_indM1=_b[M]
*
g nMPDmav_indM0=eD_MP                  if M==0
replace nMPDmav_indM0=nD_MP_mav[_n+4]  if M==1
logit D M [iw=nMPDmav_indM0]
scalar betaDMmav_indM0=_b[M]
reg D M [iw=nMPDmav_indM0]
scalar bDMmav_indM0=_b[M]
*
*
**  9a Table MD. obs. freq.Table 5.1c (=5.2c)
recode M P D (0=2)
table M D ,c(sum f)
display " OR_DM  " %6.4f exp(betaDM)
display "betaDM: " %6.4f betaDM
display " DC_DM: " %6.4f bDMols
*
** 9b Table MD (direct effect Table 5.6c)
table M D ,c(sum nD_MP_mav) row col  f(%8.3f)
display " OR_DM  " %6.4f exp(betaDMmav)
display "betaDM: " %6.4f betaDMmav
display " DC_DM: " %6.4f bDMmav_direct
*
** 9c Table MD (Black’s contribution to indirect effect)
table M D ,c(sum nMPDmav_indM1) row col  f(%8.3f)
display " OR_DM  " %6.4f exp(betaDMmav_indM1)
display "betaDM: " %6.4f betaDMmav_indM1
display " DC_DM: " %6.4f bDMmav_indM1
*
** 9d Table MD (Non-Black’s contribution to indirect effect)
table M D ,c(sum nMPDmav_indM0) row col  f(%8.3f)
display " OR_DM  " %6.4f exp(betaDMmav_indM0)
display "betaDM: " %6.4f betaDMmav_indM0
display " DC_DM: " %6.4f bDMmav_indM0
*
* sum of indirect maverick effecst
display "OR_MDmav_indM1 * OR_MDmav_indM0 = OR_MDmav_ind"
display %6.4f exp(betaDMmav_indM1) " * " %6.4f exp(betaDMmav_indM0)
display %6.4f exp(betaDM-betaDMmav)
*
display "betaMDmav_indM1 * betaMDmav_indM0 = betaMDmav_ind"
display %6.4f betaDMmav_indM1 " + " %6.4f betaDMmav_indM0
display %6.4f betaDM-betaDMmav
*
* Effect decomposition of M on D using modified (SIMMAV natural effects"
display "OR_DM = OR_DM_Dnat*OR_DM_indM1*OR_indM0"
display %6.4f exp(betaDM) " = " %6.4f exp(betaDMmav)
display "      * " %6.4f exp(betaDMmav_indM1) " * " %6.4f exp(betaDMmav_indM0)
*
display "betaDM = betaDM_mav + betaDM_indM1 + betaDM_indM0"
display  %6.4f betaDM  "  ="
display %6.4f betaDMmav " + " %6.4f betaDMmav_indM1 " + " %6.4f betaDMmav_indM0
*
display "DC_DM = DC_DM_mav + DC_DM_indM1 + DC_DM_indM0"
display  %6.4f bDMols  "  ="
display %6.4f bDMmav_direct " + " %6.4f bDMmav_indM1 " + " %6.4f bDMmav_indM0
*
***


***** Examples and Figures


*
*** chapter 5.1.1 General background 

** DAGs 
*
*** Example: DAG as recursive path diagram 
*
*  a) unrestricted (tautological) DAG: Eq. (5.1)
*
* using renamed Discrimination data 
rename (D M P)(Y X W)
recode  Y X W (2=0)
*   unconditional probabilities pXYW from table XWY
g pXYW=f/n_obs
label var pXYW 
*   = Product of pY|XW * pW|X * pX 
* saturated logit Y on X & W
logit Y c.X##c.W [fw=f]
predict pY_XxW , p
replace pY_XxW=1-pY_XxW if Y==0
* copy of probabilities  pP|M and PM in pW_X and pX
g pW_X = pP_M
g pX = pM
* right hand side of Eq. (5.1)
g eq5_1=pY_XxW*pW_X*pX
list pXYW eq5_1
drop eq5_1
*
*  b) restricted  DAG: Eq. (5.2): no direct effect of X on Y
* logit Y on W
logit Y W [fw=f]
predict pY_W ,p
replace pY_W=1-pY_W if Y==0
* right hand side of Eq. (5.2)
g eq5_2=pY_W*pW_X*pX
list pXYW eq5_2
drop eq5_2
*
*  simulated data for Eq.(5.3) see chapter3.do

** STANDARD PATH MODELS
*
*  a) moodel X -> Y Eq. (5.4)  
reg Y X [fw=f]
scalar bYX=_b[X]
*  b) model W -> X Eq. (5.5a)
reg W X [fw=f]
scalar bWX=_b[X]
*  c) moodel X -> Y <- W Eq. (5.5b)
reg Y X W [fw=f]
scalar bYX_W=_b[X]
scalar bYW_X=_b[W]
*  
display "Eq. (5.6a): bYX=bYX.W+bWX*b YW.X
display %6.4f bYX " = " %6.4f bYX_W " + " %6.4f bWX "*" %6.4f bYW_X
*
* Equation (5.6b) 
display "Eq. (5.6b): bWX * b YW.X = bYX - bYX.W"
display  %6.4f bWX*bYW_X " = "%6.4f bYX " - " %6.4f bYX_W
*
* Equation (5.6c)
cor W X [fw=f], c
scalar s2W=r(Var_1)
scalar s2X=r(Var_2)
reg Y W [fw=f]
display "Eq (5.6c): bYW = bYW.X + s²X/s²W*bWX * bYW.X
display %6.3f _b[W] " = "%6.3f  bYW_X + s2X/s2W  *  bWX * bYW_X

** RESIDUALIZED INDEPENDENT VARIABLES
*
* Equation (5.7)
*  computation of OLS_residual rW from W=bW + bWX*X +rW
reg W X [fw=f]
predict rW ,r
*  
reg Y X W [fw=f]
reg Y X   [fw=f]
*  model X -> Y <_ rW 
reg Y X rW [fw=f]
display "bYX.rW = bYX"
display _b[X] " = " bYX
display "bYrW.X = bYW.X"
display _b[rW] " =" bYW_X

* rename in original variable names of dicrimination dta
rename (X Y W)(M D P)

*** chpter 5.1.2 Comparability and Compatibility Issues 
*
*   no numeric examples, 
*      but see chapter2.do for changing logistic regression weights
*      if a maverick is added or removed from the model


*** chapter 5.2.1 Illustrating the problems: 

* LR-test between the saturated logit model (with interaction M*P)
* and the main effects only logit model
recode D M P (2=0)
logit D c.M##c.P [fw=f]
est store A
logit D M P [fw=f]
lrtest A

** ASSESSING UNDERLYING INDIRECT EFFECTS

* Equation (5.11) is not valid for logistic regression weights:
*
logit D M [fw=f]
* effect is saved as scalar betaDM
reg P M [fw=f]
* effect is saved as scalar bPMols
logit D M P [fw=f]
* effect of M is saved as scalar beta_DM_P
* effect of M is saved as scalar betaDP_M
*
display betaDM " < " betaDM_P + bPMols*betaDP_M

** EVALUATING THE SCALING RATIO φ_r⁄φ_f 

* Equation (5.12) as naive relative effect of M on D
display "betaD.P/betaDM = " betaDM_P/betaDM " = bD*M.P/bD * M phi_r/phi_f"
*
* Assuming residual variance ratio phi_r/phi_f =  0.9:"
display "corrected rel. direct effect : " %6.4f (betaDM_P/betaDM / sqrt(1/.9))


*** chapter 5.2.2 Solution: Comparing standardized effects *****

** TACKLING THE COMPARABILITY/SCALING PROBLEM 

* Equation (5.5): standard deviation od D* are computed in table 5.3 
display "Eq. (5.15): sY*full/sY*reduced = phi_r/phi_f = " %6.4f SDyDMP/SDyDM  
display "scale corrected:   " betaDM_P/betaDM / (SDyDMP/SDyDM)
display "                 = " betaDM_Phs/betaDMhs 

** TACKLING THE COMPARABILITY/SCALING PLUS THE COMPATIBILITY PROBLEM
*
* incomatibility problem:
display "different results for indirect and total D*-standard. effects:"
display "indirect effect measures:"
display %6.4f betaDMhs-betaDM_Phs " > " %6.4f bPMols * betaDP_Mhs
display "D*-stand. total effect > sum of D*-stand. direct + indirect effect" 
display %6.4f betaDMhs " > " %6.4f betaDM_Phs + bPMols*betaDM_Phs

* correct D*-stand. estimation if the full model is logistic:
display "direct effect of M on D controlled for P = " %6.4f betaDM_Phs
display "indirect effect of M on D over P = " %6.4f bDMols*betaDP_Mhs
display "total effect = " %6.4f betaDM_Phs+bDMols*betaDP_Mhs
* note: all effects are based on an unknown common scaling factor phi_f 
* for D*-standardized scale


*** chapter 5.2.3 Solution: Residualization and the KHB approach

* estimating the total effect as direct effect controlled for residualP
* note residuals of linear model  M -> P are rP_M (see table 5.r4)
* note: data from table 5.2 with same results as from table 5.1)
logit D M rP_M [iw=eD_MP]

*  KHB controlled effect: betaDM.rP = betaDM.P+bPM*betaDP_M 
*   total effect in scale phi_f
display betaDM_rP 
display _b[M] " = " betaDM_P+bPMols*betaDP_M

* using ADO khb.ado (written by u. Kohler, WZB Berlin)
* ado use individual data: expand file using variable f
expand f, g(copy)
*
khb logit D M || P
* after dependent variables (here: D) follows the exogenous predictors (here: M)
* and after "||" the mediator variables (here: P). 
* standard output is a table with: 
* a) KHB total effect(here: betaDM.rP = betaDM.P + bDP*betaDP_M)
* b) partial direct effect(here: beta_dmp),
* c) difference a) - b)
khb logit D M || P , s
* additional output from option "s":
*    ratio a)/b) and the error variance ratio (phi_r/phi_f)
khb logit D M || P , v
* additional output from option "v":
*    logistic regression outputs of full model and
*    full model with residual instead of original variables
*
* aggregate data again by dropping redundant cases
drop if copy ==1
drop copy


***** chapter 5.2.4 Solution: The path analytic approach
* see table 5.5


*** chapter 5.3 DRM *****

** 5.3.1 The main problem: confusing collapsing and mediating 

*   naive estimation 
*   indirect size effect = betaDM - betaDM.P
display "naive indirect DRM-Effect on logits: "
display "betDM - betaDM_P. " %6.4f betaDM-betaDM_P
*   naive relative indirect effect 1-betaDM.P/betaDM 
display "naive relative indirect DRM-Effect on logits: "
display "1 - " %6.4f betaDM_P/betaDM  " = " %6.4f 1-betaDM_P/betaDM
*

*** 5.3.2 Separating collapsing and mediation; the path analytic approach *****

** LINEAR-ADDITIVE RELATIONSHIPS AMONG THE INDEPENDENT VARIABLES
* using linear model M -> P for first part of path M -> P -> D
reg P M [fw=f]
display "indirect effect as lin. path M -> P * logistic path P -> D"
display " bPM * betaDP.M = " %6.4f bPMols * betaDP_M

** SYSTEMS OF LOGIT EQUATIONS
* using logistic model M -> P for first part of path M -> P -> D
logit P M [fw=f]
logit ,or
display "probability of P|M=0 = exp(betaP)/(1+exp(betaP))"
display " = " %6.4f exp(_b[_cons])/(1+exp(_b[_cons]))
*
display "probability of P|M=1 = exp(betaP+betaPM)/(1+exp(betaP+betaPM))"
display " = " %6.4f exp(_b[_cons]+betaPM)/(1+exp(_b[_cons]+betaPM))
*
display "difference = effect of M on P = first part of the indirect effect"
display " = " 0.3503-0.1417 " = same  as in linear regression" 


*** 5.3.3 Simulations and counterfactual approaches; SIMMAV and SIMNAT *****

*** SIMMAV – Simulating Mavericks
* see table 5.6 for computation and table 5.5 for effects


** SIMNAT – Simulating Pure, Natural Effects
* see table 5.7:  for computation of counterfactual tables 7b and 7d
* direct and indirect effects by analysing table 7b & 7d 
* using model M -> D (see tabls 5.5)

* direct natural DC-effects as weighted conditional linear effects M -> D
* in table 5.2a and weighted by table p(P) in table 7a
* conditional regressions
reg D M [iw=eD_MP] if P==1
scalar bDM_P1=_b[M]
reg D M [iw=eD_MP] if P==0
scalar bDM_P0=_b[M]
*weights are proportions of P=1 from freqs. of table 7a
recode D M P (0=2)
table col D ,c(sum nMPD_M1all) row col f(%8.3f)
recode D M P (2=0)
sum nMPD_M1all if P==1
display r(mean)*r(N)/n_obs
scalar w_P1=r(mean)*r(N)/n_obs
scalar w_P0=1-w_P1
* natural direct effect (if M=0 as M=1)
display "nat. direct DC all M=1 (weighted computation)"
display "   = " %6.4f w_P1*bDM_P1+w_P0*bDM_P0
* 
*same for M=1 as M=0:
* conditional regressions are the same, only weights differ
*weights are proportions of P=1 in table a) after table 7
recode D M P (0=2)
table col D ,c(sum nMPD_M0all) row col f(%8.3f)
recode D M P (2=0)
sum nMPD_M0all if P==1
display r(mean)*r(N)/n_obs
scalar w_P1=r(mean)*r(N)/n_obs
scalar w_P0=1-w_P1
* natural direct effect (if M=1 as M=0)
display "nat. direct DC all M=0 (weighted computation)"
display "   = " %6.4f w_P1*bDM_P1+w_P0*bDM_P0
* 

* counterfactul Blacks (M=1 as M=0) 
* counterfactual data tables in table 5.7 (table 7 a-d)
* natural effects:
display "direct natural logistic effect if M=1 as M=0:"
logit D M [iw=nMPD_M0all]
scalar betaDM_dnatM0=_b[M]
display "indirect natural logistic effect:"
logit D M [iw=nMPD_M0ind]
scalar betaDM_indM0=_b[M]
*
display "total (bivariate)l effect = direct + indirect ntural effects:"
display "betaDM = betaDM_Dnat + betaDM_ind"
display %6.4f betaDM " = " %6.4f betaDM_dnatM0 " + " %6.4f betaDM_indM0


***** chapter 5.4 Causal modeling 

*** Simulated data for Figure 5.3
* Y5.3a*-1+0.6*X+0.5*W5.3a      +U
* Y5.3b*=1.0.6*X+0.5*W5.3b-0.5*T+U_
* X=0.7*S+E_s
* W5.3a=0.5*S+0.5*X+E_w5.3a
* W5.3b=0.5*T+0.5*X+E_w5.3b
* Y5.3a=1 if Y5.3a* > 0 else 0
* Y5.3b=1 if Y5.3b* > 0 else 0
* S, T, X, W are standard normal distributed
* U is standard logistic distributed
* n=5000
*
* data generation
clear
set obs 5000
* generating standard logistic U uncorretated with 4 indpendent N(0/1) 
*                                                  variables Q S T X W 
set seed 987654321
* U
g u=rlogistic()
sum u
replace u=(u-r(mean))/r(sd)*_pi/sqrt(3)
sum u
* S
g s=rnormal()
reg s u
predict tmp ,r
sum tmp
replace s=(tmp-r(mean))/r(sd)
drop tmp
* T
g t=rnormal()
reg t s u
predict tmp ,r
sum tmp
replace t=(tmp-r(mean))/r(sd)
drop tmp
* temporary W
g w=rnormal()
reg w t s u
predict tmp ,r
sum tmp
replace w=(tmp-r(mean))/r(sd)
drop tmp
* X
g x=rnormal()
reg x w t s u
predict tmp ,r
sum tmp
replace x=(tmp-r(mean))/r(sd)
drop tmp
* relate variables by regression euqarions
* X=0.7*S+sqrt(0.51)*X
replace x=0.7*s+sqrt(0.51)*x
* Different W in fig 5.3a and 5.3b
g w53a=0.5*s+0.5*x+sqrt(0.15)*w
g w53b=0.5*x+0.5*t+sqrt(0.5)*w
drop w
* 
g y53a_star=1+0.5*x+0.5*w53a+u
g y53b_star=1+0.5*x+0.5*w53b-0.5*t+u
*
* means and covariances of the data:
corr * ,m c
* dichotomisation of Y*
g y53a=(y53a_star>0)
g y53b=(y53b_star>0)
*
tab1 y53a y53b
* standard regression on true models
reg y53a_star x w53a 
reg w53a x s
reg x s
*
reg y53b_star x w53b t
reg w53b x t
* logistic regression on y53a & y53b (=true models)
logit y53a x w53a 
*
logit y53b x w53b t
*

* consequences of omitting S in fig. 5.3a
* standard regression:
reg y53a_star x w53a
* -> correct estimation because S is blocked by W in bYX.W and by X in bYW.X
*
reg y53a_star x
* -> total effect is biased due to confounder S (true effect=0.5+0.5²=.75)
*
reg w53a x
* -> direct effect is biased due confounder S (true effect=0.5)
*
*
* logistic regression:
logit y53a x w53a
* -> correct estimation of DRM
*
logit y53a x
* -> total effect is biased due to confounder S
*

* consequences of omitting T in fig. 5.3b
* standard regression:
reg y53b_star x w53b
* -> effects of X and W are biased due confounder T (true effects: 0.5 both)
*
reg y53b_star x
reg y53b_star x t
* -> total effect of X is correct because T is a maverick with respect to X   
*
* logistic regression:
logit y53b x w53b
* -> effects of X and W are biased due confounder T (true effects: 0.5 both)
*
logit y53b x
logit y53b x t
* -> total effect of X is correct because T is a maverick with respect to X
*    but has a small negative collapsing effect due to ignoring maverick T

log close _all
